local _,stash = ...

-- for throttling purposes (if multiple events happen during the duration, reset wait with each event)
stash.timer = CreateFrame("Frame")
stash.timer:Hide()
stash.timer.times = {} -- indexed by function, the duration to wait before running the function
stash.timer.running = {} -- ordered list of functions waiting to run
stash.timer.selves = {} -- indexed by function, the 'self' to pass as the first argument of the function

-- after given duration (seconds), the func will run, with funcSelf as 'self' if given (without a self reference if not!)
function stash.timer:Start(duration,func,funcSelf)
    self.times[func] = duration
    self.selves[func] = funcSelf
    if not tContains(self.running,func) then
        tinsert(self.running,func)
    end
    self:Show()
end

-- stops a timer running
function stash.timer:Stop(func)
    for i=#self.running,1,-1 do
        if self.running[i]==func then
            tremove(self.running,i)
        end
    end
end

-- OnUpdate for the timer that ticks down running timers
stash.timer:SetScript("OnUpdate",function(self,elapsed)
    local tick = false
    for i=#self.running,1,-1 do
        local func = self.running[i]
        self.times[func] = self.times[func] - elapsed
        if self.times[func] < 0 then
            tremove(self.running,i)
            func(self.selves[func]) -- run the function with its saved reference if one given
        end
        tick = true
    end
    -- if everything was removed from self.running in previous frame, tick will not be true
    if not tick then
        self:Hide()
    end
end)


